// DoCreateDialogTooltip - creates a tooltip control for a dialog box, 
//     enumerates the child control windows, and installs a hook 
//     procedure to monitor the message stream for mouse messages posted 
//     to the control windows. 
// Returns TRUE if successful, or FALSE otherwise. 
// 
// Global variables 
// g_hinst - handle to the application instance 
// g_hwndTT - handle to the tooltip control 
// g_hwndDlg - handle to the dialog box 
// g_hhk - handle to the hook procedure 
#include <windows.h> 
#include <windowsx.h>
#include <commctrl.h>
#include <string.h>
#include <winbase.h>
#include "resource.h"

BYTE RegEaxM1[9],RegEbxM1[9],RegEcxM1[9],RegEdxM1[9],RegEdiM1[9],RegEsiM1[9],RegAux1M1[9],RegAux2M1[9],RegRecivedM1[9];
BYTE RegEaxM2[9],RegEbxM2[9],RegEcxM2[9],RegEdxM2[9],RegEdiM2[9],RegEsiM2[9],RegAux1M2[9],RegAux2M2[9],RegRecivedM2[9];
BYTE RegEaxM3[9],RegEbxM3[9],RegEcxM3[9],RegEdxM3[9],RegEdiM3[9],RegEsiM3[9],RegAux1M3[9],RegAux2M3[9],RegRecivedM3[9];
BYTE RegEaxM4[9],RegEbxM4[9],RegEcxM4[9],RegEdxM4[9],RegEdiM4[9],RegEsiM4[9],RegAux1M4[9],RegAux2M4[9],RegRecivedM4[9];





VOID CreateToolTipString(char RegString[130],int Flag);
BOOL EnumChildProc(HWND hwndCtrl, LPARAM lParam); 
LRESULT CALLBACK GetMsgProc(int nCode, WPARAM wParam, LPARAM lParam);

HWND g_hwndTT,g_hwndDlg,g_hhk;
HINSTANCE g_hinst;

HINSTANCE   hComCtl;
DWORD address;


BOOL DoCreateDialogTooltip(void) 
{ 
 
    hComCtl = LoadLibrary("comctl32.dll");
	address =GetProcAddress(hComCtl,"InitCommonControls");
	// Ensure that the common control DLL is loaded, and create 
    // a tooltip control. 
     __asm {
		call dword ptr [address];
	};//chiama InitCommonControls() dato che la microsoft ha deciso 
	 //di farmi impazzire con versioni differenti 
	 //della stessa libreria non compatibili
    
	 
	 g_hwndTT = CreateWindowEx(0, TOOLTIPS_CLASS, (LPSTR) NULL,TTS_ALWAYSTIP, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, g_hwndDlg, (HMENU) NULL, g_hinst, NULL); 
 
    if (g_hwndTT == NULL) 
        return FALSE; 
 
    // Enumerate the child windows to register them with the tooltip
    // control. 
    if (!EnumChildWindows(g_hwndDlg, (WNDENUMPROC) EnumChildProc, 0)) 
        return FALSE; 
 
    // Install a hook procedure to monitor the message stream for mouse 
    // messages intended for the controls in the dialog box. 
    g_hhk = SetWindowsHookEx(WH_GETMESSAGE, GetMsgProc, (HINSTANCE) NULL, GetCurrentThreadId()); 
 
    if (g_hhk == (HHOOK) NULL) 
        return FALSE; 
 
    return TRUE; 
} 
 
// EnumChildProc - registers control windows with a tooltip control by
//     using the TTM_ADDTOOL message to pass the address of a 
//     TOOLINFO structure. 
// Returns TRUE if successful, or FALSE otherwise.
//
// hwndCtrl - handle of a control window 
// lParam - application-defined value (not used) 
BOOL EnumChildProc(HWND hwndCtrl, LPARAM lParam) 
{ 
    TOOLINFO ti; 
    char szClass[64]; 
 
    // Skip static controls. 
    GetClassName(hwndCtrl, szClass, sizeof(szClass)); 
    if (lstrcmpi(szClass, "STATIC")) { 
        ti.cbSize = sizeof(TOOLINFO); 
        ti.uFlags = TTF_IDISHWND; 
        ti.hwnd = g_hwndDlg; 
        ti.uId = (UINT) hwndCtrl; 
        ti.hinst = 0; 
        ti.lpszText = LPSTR_TEXTCALLBACK; 
        SendMessage(g_hwndTT, TTM_ADDTOOL, 0, 
            (LPARAM) (LPTOOLINFO) &ti); 
    } 
    return TRUE;
	} 
 
// GetMsgProc - monitors the message stream for mouse messages intended 
//     for a control window in the dialog box. 
// Returns a message-dependent value. 
//
// nCode - hook code 
// wParam - message flag (not used) 
// lParam - address of an MSG structure 
LRESULT CALLBACK GetMsgProc(int nCode, WPARAM wParam, LPARAM lParam) 
{ 
    MSG *lpmsg; 
 
    lpmsg = (MSG *) lParam; 
    if (nCode < 0 || !(IsChild(g_hwndDlg, lpmsg->hwnd))) 
        return (CallNextHookEx(g_hhk, nCode, wParam, lParam)); 
 
    switch (lpmsg->message) { 
        case WM_MOUSEMOVE: 
        case WM_LBUTTONDOWN: 
        case WM_LBUTTONUP: 
        case WM_RBUTTONDOWN: 
        case WM_RBUTTONUP: 
            if (g_hwndTT != NULL) { 
                MSG msg; 
 
                msg.lParam = lpmsg->lParam; 
                msg.wParam = lpmsg->wParam; 
                msg.message = lpmsg->message; 
                msg.hwnd = lpmsg->hwnd; 
                SendMessage(g_hwndTT, TTM_RELAYEVENT, 0, 
                    (LPARAM) (LPMSG) &msg); 
            } 
            break; 
        default: 
            break; 
    } 
    return (CallNextHookEx(g_hhk, nCode, wParam, lParam)); 
} 
 
// OnWMNotify - provides the tooltip control with the appropriate text 
//     to display for a control window. This function is called by 
//     the dialog box procedure in response to a WM_NOTIFY message. 
//
// lParam - second message parameter of the WM_NOTIFY message. 
VOID OnWMNotify(LPARAM lParam) 
{ 
    LPTOOLTIPTEXT lpttt; 
    int idCtrl; 
	HWND hwndF;
    char RegString[130]="";
	
	if ((((LPNMHDR) lParam)->code) ==  TTN_GETDISPINFO) {
		idCtrl = GetDlgCtrlID((HWND) ((LPNMHDR) lParam)->idFrom);
		lpttt = (LPNMTTDISPINFO)lParam;
				__asm{ 
						push eax;
							mov eax,[lParam];
							mov eax,[eax];
							mov hwndF,eax;//cosa non si fa per pigliare un handle
							pop eax;//microsoft :((((((
						}	
				
				//lparam->hwndForm non funzia (maledette conversioni)
		
		switch (idCtrl) { 
		case IDR1: {
					CreateToolTipString(RegString,1);
					SendMessage(hwndF, TTM_SETMAXTIPWIDTH, 0, 300);
					};break;
		
		case IDR2: {
					CreateToolTipString(RegString,2);
					SendMessage(hwndF, TTM_SETMAXTIPWIDTH, 0, 300);
					};break;
		
		case IDR3: {
					CreateToolTipString(RegString,3);
					SendMessage(hwndF, TTM_SETMAXTIPWIDTH, 0, 300);
					};break;
		
		case IDR4: {
					CreateToolTipString(RegString,4);
					SendMessage(hwndF, TTM_SETMAXTIPWIDTH, 0, 300);
					};break;
		}		
				
				lpttt->lpszText = RegString;
                return; 
				 
 
	}

return; 
} 

VOID CreateToolTipString(char RegString[130],int Flag)
{
	int RegStringLen=0;

	switch (Flag) {
				case 1:{
					lstrcat(RegString,"Eax=");
					lstrcat(RegString,RegEaxM1);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';
				
					lstrcat(RegString,"Ebx=");
					lstrcat(RegString,RegEbxM1);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';
					   
					lstrcat(RegString,"Ecx=");
					lstrcat(RegString,RegEcxM1);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';
				
					lstrcat(RegString,"Edx=");
					lstrcat(RegString,RegEdxM1);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';   
					   
					lstrcat(RegString,"Edi=");
					lstrcat(RegString,RegEdiM1);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';
				
					lstrcat(RegString,"Esi=");
					lstrcat(RegString,RegEsiM1);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';   
					   
					lstrcat(RegString,"Aux1=");
					lstrcat(RegString,RegAux1M1);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';
				
					lstrcat(RegString,"Aux2=");
					lstrcat(RegString,RegAux2M1);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';   
					   
					lstrcat(RegString,"Recived=");
					lstrcat(RegString,RegRecivedM1);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\00';
					};break;				
				
				
				case 2:{
					lstrcat(RegString,"Eax=");
					lstrcat(RegString,RegEaxM2);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';
				
					lstrcat(RegString,"Ebx=");
					lstrcat(RegString,RegEbxM2);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';
					   
					lstrcat(RegString,"Ecx=");
					lstrcat(RegString,RegEcxM2);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';
				
					lstrcat(RegString,"Edx=");
					lstrcat(RegString,RegEdxM2);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';   
					   
					lstrcat(RegString,"Edi=");
					lstrcat(RegString,RegEdiM2);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';
				
					lstrcat(RegString,"Esi=");
					lstrcat(RegString,RegEsiM2);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';   
					   
					lstrcat(RegString,"Aux1=");
					lstrcat(RegString,RegAux1M2);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';
				
					lstrcat(RegString,"Aux2=");
					lstrcat(RegString,RegAux2M2);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';   
					   
					lstrcat(RegString,"Recived=");
					lstrcat(RegString,RegRecivedM2);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\00';
					};break;	
				
				case 3:{
					lstrcat(RegString,"Eax=");
					lstrcat(RegString,RegEaxM3);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';
				
					lstrcat(RegString,"Ebx=");
					lstrcat(RegString,RegEbxM3);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';
					   
					lstrcat(RegString,"Ecx=");
					lstrcat(RegString,RegEcxM3);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';
				
					lstrcat(RegString,"Edx=");
					lstrcat(RegString,RegEdxM3);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';   
					   
					lstrcat(RegString,"Edi=");
					lstrcat(RegString,RegEdiM3);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';
				
					lstrcat(RegString,"Esi=");
					lstrcat(RegString,RegEsiM3);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';   
					   
					lstrcat(RegString,"Aux1=");
					lstrcat(RegString,RegAux1M3);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';
				
					lstrcat(RegString,"Aux2=");
					lstrcat(RegString,RegAux2M3);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';   
					   
					lstrcat(RegString,"Recived=");
					lstrcat(RegString,RegRecivedM3);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\00';
					};break;	
				
				case 4:{
					lstrcat(RegString,"Eax=");
					lstrcat(RegString,RegEaxM4);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';
				
					lstrcat(RegString,"Ebx=");
					lstrcat(RegString,RegEbxM4);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';
					   
					lstrcat(RegString,"Ecx=");
					lstrcat(RegString,RegEcxM4);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';
				
					lstrcat(RegString,"Edx=");
					lstrcat(RegString,RegEdxM4);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';   
					   
					lstrcat(RegString,"Edi=");
					lstrcat(RegString,RegEdiM4);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';
				
					lstrcat(RegString,"Esi=");
					lstrcat(RegString,RegEsiM4);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';   
					   
					lstrcat(RegString,"Aux1=");
					lstrcat(RegString,RegAux1M4);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';
				
					lstrcat(RegString,"Aux2=");
					lstrcat(RegString,RegAux2M4);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\r';
					RegString[RegStringLen]='\n';   
					   
					lstrcat(RegString,"Recived=");
					lstrcat(RegString,RegRecivedM4);
					RegStringLen=lstrlen(RegString);
					RegString[RegStringLen]='\00';
					};break;	
				
};
}
